// 
// Much code borrowed from "VolumeZones" indicator, Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// Modified to display rolling Market Profile by Ben L, sbgtrading@yahoo.com
//  The profile is calculated from the bars currently displayed on the chart.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Collections.Generic;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

//=====================================================================================================
//   OpenSource code distributed under the CDDL v1.0 (http://www.opensource.org/licenses/cddl1.php)
//=====================================================================================================
//
//  Enjoy this OpenSource NinjaScript v6.0 indicator!  May it help bring you great amounts of profit!

//  If you have profited from this code, then please consider a suitable donation of any amount to its 
//  Initial Developer and/or other Contributors.  Contact information given below.

//  Also, feel free to email enhancement suggestions to:
//          Initial Developer:  SBG Trading Corp., Ben Letto, sbgtrading@yahoo.com February 2, 2008
//
//=====================================================================================================

namespace NinjaTrader.Indicator
{	
	/// <summary>
	/// Each histo bar represents the number of times that price was touched or gapped over.
	/// </summary>
	[Description("Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.")]
	public class CalculateValueAreaHisto : Indicator
	{
		#region Variables
		private double[] MP = new double [1500]; 

		private SolidBrush		histoBrush;
		private SolidBrush		POCBrush;
		private SolidBrush		VABrush;
		private Color			POCcolor		= Color.Red;
		private Color			histoColor		= Color.Gold;
		private Color			VAcolor			= Color.Cyan;
		private int				barSpacing		= 1;
		private int				ProfileID		= 1;
		private string			profileName		= "TPO";
		private int				transparency	= 80;
		private double			pctOfWidth      = 0.80;
		private double			AvgPrice		= 0.0;
		private double			HitsTotal		= 0.0;
		private double			pctInVA			= 0.7;
		private double			VAtop,VAbot;
		private double 			ChartHighPrice,ChartLowPrice,PricePtr,volumeMax;
		private int 			PriceDigits,firstBar,lastBar;
		private double			thepoc = 0.0;
		private string FormatString;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Overlay					= true;
			ZOrder					= -1;
			CalculateOnBarClose		= false;
			PriceDigits = Math.Max(0,TickSize.ToString().Length-2);
			if(PriceDigits==0) FormatString="0";
			if(PriceDigits==1) FormatString="0.0";
			if(PriceDigits==2) FormatString="0.00";
			if(PriceDigits==3) FormatString="0.000";
			if(PriceDigits==4) FormatString="0.0000";
			if(PriceDigits>=5) FormatString="0.00000";
			RecalculateColors();
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
		}

		private int GetYPos(double price, Rectangle bounds, double min, double max)
		{
			return (int) ((bounds.Y + bounds.Height) - 
				((price - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height);
		}

		/// <summary>
		/// Called when the indicator is plotted.
		/// </summary>
		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			lastBar		= Math.Min(ChartControl.LastBarPainted, Bars.Count - 1);
			firstBar	= (lastBar - ChartControl.BarsPainted) + 1;
			int i;
			
			// Find highest and lowest price points
			ChartHighPrice = 0.0;
			ChartLowPrice  = double.MaxValue;
			for (i = firstBar; i <= lastBar && i >= 0; i++)
			{
				ChartHighPrice = Math.Max(ChartHighPrice, Bars.Get(i).High);
				ChartLowPrice  = Math.Min(ChartLowPrice , Bars.Get(i).Low);
			}

			for(i=0;i<MP.Length;i++) MP[i]=0.0;

			if((ChartHighPrice-ChartLowPrice)/TickSize+2 >= MP.Length) 
			{	ChartHighPrice = (MP.Length-1)*TickSize+ChartLowPrice;
				DrawText("ErrorText","Array out of bounds:  Cannot display histo data above "+ChartHighPrice.ToString()+" price level.  HIT F5 to re-initialize",0,Close[0],Color.Red);
			}

			if(ProfileID==1) GenerateProfile_VOC();
			if(ProfileID==2) GenerateProfile_TPO();
			if(ProfileID==3) GenerateProfile_VWTPO();
			AvgPrice = NormalizePrice(CalculateAvgPrice());
			int TicksInVA = CalculateTicksForValueArea();
			VAtop = AvgPrice+TicksInVA*TickSize;
			VAbot = AvgPrice-TicksInVA*TickSize;
			
			// Pass 2: Paint the histo bars
			i=0;
			PricePtr = ChartLowPrice;
			int MaxBarWidth = (int) ((bounds.Width * pctOfWidth));
			int yUpper = GetYPos(VAtop, bounds, min, max) + barSpacing;
			int yLower = GetYPos(VAbot, bounds, min, max);
			graphics.FillRectangle (VABrush, new Rectangle(
									bounds.X, yUpper, 
									MaxBarWidth, Math.Abs(yUpper - yLower)));

			while(PricePtr<=ChartHighPrice)
			{
				yUpper = GetYPos(PricePtr+TickSize/2.0, bounds, min, max) + barSpacing;
				yLower = GetYPos(PricePtr-TickSize/2.0, bounds, min, max);

				double x=0.0;
				x = MP[(int) Math.Round((PricePtr-ChartLowPrice)/TickSize,0) + 1];
				int barWidth   = (int) ((bounds.Width * pctOfWidth) * (x / volumeMax));

				if(x == volumeMax)
				{
					thepoc = PricePtr;
					graphics.FillRectangle(POCBrush, new Rectangle(
						bounds.X, yUpper, 
						barWidth,	Math.Abs(yUpper - yLower)));
					
				} else if(PricePtr<=VAtop && PricePtr>=VAbot)
				{
					graphics.FillRectangle(histoBrush, new Rectangle(
						bounds.X, yUpper, 
						barWidth,	Math.Abs(yUpper - yLower)));
				}
				else
				{
					graphics.FillRectangle(histoBrush, new Rectangle(
						bounds.X, yUpper, 
						barWidth,	Math.Abs(yUpper - yLower)));
				}
				PricePtr = PricePtr + TickSize;
			}
		}

		private void GenerateProfile_VOC()
		{	volumeMax = 0.0;
			for (int idx = firstBar; idx <= lastBar; idx++)
			{
				IBar bar = Bars.Get(idx);
				double PricePtr = Math.Round(bar.Close,PriceDigits);
				int i = (int) Math.Round((PricePtr-ChartLowPrice)/TickSize,0) + 1;
				MP[i] = MP[i]+bar.Volume;
				volumeMax = Math.Max(volumeMax,MP[i]);
			}
		}
		private void GenerateProfile_TPO()
		{	volumeMax = 0.0;
			for (int idx = firstBar; idx <= lastBar; idx++)
			{
				IBar bar = Bars.Get(idx);
				double H=Math.Round(bar.High,PriceDigits);
				double L=Math.Round(bar.Low,PriceDigits);
				double PricePtr = L;
				while(PricePtr<=H)
				{	int i = (int) Math.Round((PricePtr-ChartLowPrice)/TickSize,0) + 1;
					MP[i] = MP[i]+1.0;
					volumeMax = Math.Max(volumeMax,MP[i]);
					PricePtr = PricePtr+TickSize;
				}
			}
		}
		private void GenerateProfile_VWTPO()
		{	volumeMax = 0.0;
			for (int idx = firstBar; idx <= lastBar; idx++)
			{
				IBar bar = Bars.Get(idx);
				double H=Math.Round(bar.High,PriceDigits);
				double L=Math.Round(bar.Low,PriceDigits);
				double PricePtr = L;
				double V = bar.Volume/((H-L)/TickSize+1);
				while(PricePtr <= H)
				{	int i = (int) Math.Round((PricePtr-ChartLowPrice)/TickSize,0) + 1;
					MP[i] = MP[i]+V;
					volumeMax = Math.Max(volumeMax,MP[i]);
					PricePtr = PricePtr+TickSize;
				}
			}
		}
		private double CalculateAvgPrice()
		{	//Calculate the Average price as weighted by the hit counts
			double THxP=0.0; //Total of Hits multiplied by Price at that volume
			double PricePtr = ChartLowPrice;
			HitsTotal = 0.0;
			while(PricePtr <= ChartHighPrice)
			{	double x = MP[(int) Math.Round((PricePtr-ChartLowPrice)/TickSize,0) + 1];
				THxP = THxP + x * PricePtr;
				HitsTotal = HitsTotal + x;
				PricePtr = PricePtr + TickSize;
			}
			return(THxP/HitsTotal);
		}
		private int CalculateTicksForValueArea()
		{	int AvgPricePtr = (int) Math.Round((AvgPrice-ChartLowPrice)/TickSize,0)+1;
			int Ticks = 0;
			double HitsIncluded = MP[AvgPricePtr];
			while(HitsIncluded/HitsTotal < pctInVA)
			{	Ticks++;
				HitsIncluded = HitsIncluded + MP[AvgPricePtr+Ticks] + MP[AvgPricePtr-Ticks];
			}
			return(Ticks);
		}
			
		private void RecalculateColors() 
		{
			int	alpha	= (int) (255.0 * ((100.0 - transparency) / 100.0));

			histoBrush  = new SolidBrush(Color.FromArgb(alpha, histoColor.R, histoColor.G, histoColor.B));
			POCBrush    = new SolidBrush(Color.FromArgb(alpha, POCcolor.R, POCcolor.G, POCcolor.B));
			VABrush     = new SolidBrush(Color.FromArgb(alpha, VAcolor.R, VAcolor.G, VAcolor.B));
		}
		
		private double NormalizePrice(double p)
		{	double result = Math.Round(p/TickSize,0)*TickSize;
			return(result);
		}
		#region Properties
		
		[Browsable(false)]
        [XmlIgnore()]
        public double ThePOC
        {
			// We need to call the Update() method to ensure our exposed variable is in up-to-date.
            get { Update(); return thepoc; }
        }

		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of VA bars.")]
		[Category("Colors")]
		[Gui.Design.DisplayNameAttribute("Value Area color")]
		public Color VAColor
		{
			get { return VAcolor; }
			set { VAcolor = value; RecalculateColors(); }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string VAColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(VAcolor); }
			set { VAcolor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of histogram bars.")]
		[Category("Colors")]
		[Gui.Design.DisplayNameAttribute("Histo color")]
		public Color HistoColor
		{
			get { return histoColor; }
			set { histoColor = value; RecalculateColors(); }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string HistoColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(histoColor); }
			set { histoColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[XmlIgnore()]
		[Description("Color of POC level.")]
		[Category("Colors")]
		[Gui.Design.DisplayNameAttribute("POC color")]
		public Color poc_color
		{
			get { return POCcolor; }
			set { POCcolor = value; RecalculateColors(); }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string POCColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(POCcolor); }
			set { POCcolor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}

		/// <summary>
		/// </summary>
		[Description("The percentage of transparency of the histo bars.")]
		[Category("Parameters")]
		public int Transparency
		{
			get { return transparency; }
			set 
			{ 
				transparency = Math.Max(0 , value); 
				transparency = Math.Min(90, value);
			}
		}
		[Description("The percentage of Volume in Value Area")]
		[Category("Parameters")]
		public double PctInVA
		{
			get { return pctInVA; }
			set 
			{ 
				pctInVA = Math.Max(0.0, value); 
				pctInVA = Math.Min(1.0, value);
			}
		}
		/// <summary>
		/// </summary>
		[Description("Valid profile types: VOC (volume on close), TPO (time price opportunities) or VWTPO (volume weighted TPO).")]
		[Category("Parameters")]
		public string ProfileName
		{
			get { return profileName.ToUpper(); }
			set 
			{ 
				profileName = value.ToUpper(); 
				if(profileName=="VOC") ProfileID=1;
				if(profileName=="TPO") ProfileID=2;
				if(profileName=="VWTPO") ProfileID=3;
			}
		}
		/// <summary>
		/// </summary>
		[Description("The percentage screen width consumed by histo")]
		[Category("Parameters")]
		public double PctOfWidth
		{
			get { return pctOfWidth; }
			set 
			{ 
				pctOfWidth = Math.Max(0.10 , value); 
				pctOfWidth = Math.Min(1.0, value);
			}
		}
#endregion
	}
}


#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CalculateValueAreaHisto[] cacheCalculateValueAreaHisto = null;

        private static CalculateValueAreaHisto checkCalculateValueAreaHisto = new CalculateValueAreaHisto();

        /// <summary>
        /// Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.
        /// </summary>
        /// <returns></returns>
        public CalculateValueAreaHisto CalculateValueAreaHisto(double pctInVA, double pctOfWidth, string profileName, int transparency)
        {
            return CalculateValueAreaHisto(Input, pctInVA, pctOfWidth, profileName, transparency);
        }

        /// <summary>
        /// Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.
        /// </summary>
        /// <returns></returns>
        public CalculateValueAreaHisto CalculateValueAreaHisto(Data.IDataSeries input, double pctInVA, double pctOfWidth, string profileName, int transparency)
        {
            checkCalculateValueAreaHisto.PctInVA = pctInVA;
            pctInVA = checkCalculateValueAreaHisto.PctInVA;
            checkCalculateValueAreaHisto.PctOfWidth = pctOfWidth;
            pctOfWidth = checkCalculateValueAreaHisto.PctOfWidth;
            checkCalculateValueAreaHisto.ProfileName = profileName;
            profileName = checkCalculateValueAreaHisto.ProfileName;
            checkCalculateValueAreaHisto.Transparency = transparency;
            transparency = checkCalculateValueAreaHisto.Transparency;

            if (cacheCalculateValueAreaHisto != null)
                for (int idx = 0; idx < cacheCalculateValueAreaHisto.Length; idx++)
                    if (Math.Abs(cacheCalculateValueAreaHisto[idx].PctInVA - pctInVA) <= double.Epsilon && Math.Abs(cacheCalculateValueAreaHisto[idx].PctOfWidth - pctOfWidth) <= double.Epsilon && cacheCalculateValueAreaHisto[idx].ProfileName == profileName && cacheCalculateValueAreaHisto[idx].Transparency == transparency && cacheCalculateValueAreaHisto[idx].EqualsInput(input))
                        return cacheCalculateValueAreaHisto[idx];

            CalculateValueAreaHisto indicator = new CalculateValueAreaHisto();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.PctInVA = pctInVA;
            indicator.PctOfWidth = pctOfWidth;
            indicator.ProfileName = profileName;
            indicator.Transparency = transparency;

            CalculateValueAreaHisto[] tmp = new CalculateValueAreaHisto[cacheCalculateValueAreaHisto == null ? 1 : cacheCalculateValueAreaHisto.Length + 1];
            if (cacheCalculateValueAreaHisto != null)
                cacheCalculateValueAreaHisto.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCalculateValueAreaHisto = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CalculateValueAreaHisto CalculateValueAreaHisto(double pctInVA, double pctOfWidth, string profileName, int transparency)
        {
            return _indicator.CalculateValueAreaHisto(Input, pctInVA, pctOfWidth, profileName, transparency);
        }

        /// <summary>
        /// Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.
        /// </summary>
        /// <returns></returns>
        public Indicator.CalculateValueAreaHisto CalculateValueAreaHisto(Data.IDataSeries input, double pctInVA, double pctOfWidth, string profileName, int transparency)
        {
            return _indicator.CalculateValueAreaHisto(input, pctInVA, pctOfWidth, profileName, transparency);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CalculateValueAreaHisto CalculateValueAreaHisto(double pctInVA, double pctOfWidth, string profileName, int transparency)
        {
            return _indicator.CalculateValueAreaHisto(Input, pctInVA, pctOfWidth, profileName, transparency);
        }

        /// <summary>
        /// Volume Zones plots a horizontal histogram that overlays a price chart. The histogram bars stretch from left to right starting at the left side of the chart. The length of each bar is determined by the cumulative total of all volume bars for the periods during which the price fell within the vertical range of the histogram bar.
        /// </summary>
        /// <returns></returns>
        public Indicator.CalculateValueAreaHisto CalculateValueAreaHisto(Data.IDataSeries input, double pctInVA, double pctOfWidth, string profileName, int transparency)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CalculateValueAreaHisto(input, pctInVA, pctOfWidth, profileName, transparency);
        }

    }
}
#endregion
